/*=========================================================================

  Program:   Visualization Toolkit
  Module:    TestLinearCellExtrusion2.cxx

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// This tests vtkLinearCellExtrusionFilter

#include "vtkDataSetSurfaceFilter.h"
#include "vtkElevationFilter.h"
#include "vtkLinearCellExtrusionFilter.h"
#include "vtkNew.h"
#include "vtkPolyDataMapper.h"
#include "vtkPolyDataNormals.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderer.h"
#include "vtkSphereSource.h"

//------------------------------------------------------------------------------
int TestLinearCellExtrusion2(int vtkNotUsed(argc), char* vtkNotUsed(argv)[])
{
  vtkNew<vtkSphereSource> sphere;
  sphere->SetThetaResolution(11);
  sphere->SetPhiResolution(11);
  sphere->SetRadius(1.0);
  sphere->Update();

  vtkNew<vtkElevationFilter> ele;
  ele->SetInputConnection(sphere->GetOutputPort());
  ele->SetLowPoint(-0.8, 0.0, 0.0);
  ele->SetHighPoint(1.0, 0, 0);

  vtkNew<vtkPolyDataNormals> normals;
  normals->SetInputConnection(ele->GetOutputPort());
  normals->ComputePointNormalsOn();
  normals->ComputeCellNormalsOff();
  normals->ConsistencyOn();
  normals->SplittingOff();

  vtkNew<vtkLinearCellExtrusionFilter> extrusionCellNormal;
  extrusionCellNormal->SetInputConnection(normals->GetOutputPort());
  extrusionCellNormal->SetNormalsTypeToCellNormals();
  extrusionCellNormal->SetScaleModeToDataScalingOff();
  extrusionCellNormal->SetScaleFactor(0.5);
  extrusionCellNormal->Update();

  vtkNew<vtkLinearCellExtrusionFilter> extrusionPointNormal;
  extrusionPointNormal->SetInputConnection(normals->GetOutputPort());
  extrusionPointNormal->SetNormalsTypeToPointNormals();
  extrusionPointNormal->SetScaleModeToScaleByScalar();
  extrusionPointNormal->SetInputArrayToProcess(0, 0, 0,
    vtkDataObject::FIELD_ASSOCIATION_POINTS, "Elevation");
  extrusionPointNormal->SetScaleFactor(1.0);
  extrusionPointNormal->MergeDuplicatePointsOn();
  extrusionPointNormal->Update();

  vtkNew<vtkDataSetSurfaceFilter> surf2;
  surf2->SetInputConnection(extrusionCellNormal->GetOutputPort());

  vtkNew<vtkDataSetSurfaceFilter> surf3;
  surf3->SetInputConnection(extrusionPointNormal->GetOutputPort());

  vtkNew<vtkPolyDataMapper> mapper;
  mapper->SetInputConnection(surf2->GetOutputPort());

  vtkNew<vtkPolyDataMapper> mapperUser;
  mapperUser->SetInputConnection(surf3->GetOutputPort());
  mapperUser->SelectColorArray("Elevation");
  mapperUser->SetScalarRange(0.0, 1.0);
  mapperUser->SetColorModeToMapScalars();

  vtkNew<vtkActor> actor;
  actor->SetMapper(mapper);

  vtkNew<vtkActor> actorUser;
  actorUser->SetPosition(0.0, 3.5, 0.0);
  actorUser->SetMapper(mapperUser);

  vtkNew<vtkRenderer> ren;
  ren->AddActor(actor);
  ren->AddActor(actorUser);

  vtkNew<vtkRenderWindow> renWin;
  renWin->AddRenderer(ren);
  renWin->SetSize(300, 300);
  renWin->SetMultiSamples(0);

  vtkNew<vtkRenderWindowInteractor> iren;
  iren->SetRenderWindow(renWin);

  // Render and test
  renWin->Render();

  iren->Start();

  return 0;
}
